%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%      MAIN program :
%%              MainMCVEM.m
%%
%%  Last Modification : August, 16 2005.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%      ATTENTION : CONVENTION SUR BETA :
%%
%%  ln prior(z;beta) = - H(z;beta) - ln W(beta)
%%  avec H(z;beta)  = - 0.5 Beta sum_k  sum_{j in N(k)} z_k' z_j
%%                  = - Beta sum_{k ~ j}  z_k' z_j
%%  et Beta > 0.
%%  Voisinage : 4 ou 8 connexes ; potentiel de clique d'ordre 2.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 

%%  1/ Load the data
%%%%%%%%%%%%%%%%%%%%%
    %%  1/ TRIANGLE
%     load ~/INRIA/MCVEM_publi/Example/triangle05.dat;
%     ObsPict=triangle05;
%     clear triangle05;
%     load ~/INRIA/MCVEM_publi/Example/triangle05.true;
%     TruePict=triangle05;    % {0, ..., G-1}-valued.
%     clear triangle05;
%     TruePict=TruePict+1;    % {1, ..., G}-valued.
%     G=4;
%     NbrNeigh='4-Neigh';
%     [I J]=size(ObsPict); 
%     load ~/INRIA/MCVEM_publi/Simulations/Triangle/Triangle05.init
%     Partition=reshape(Triangle05,I*J,1);
  
%     
    
    %% 2/   MICKEY
%      load ../Example/mickbin03.dat;
%      ObsPict=mickbin03';
%      clear mickbin03;
%      load ../Example/mickbin05.true;
%      TruePict=flipud(mickbin05);
%      clear mickbin05;
%      G=2;
%      NbrNeigh='8-Neigh';     
     
     
%     %%  3/  LOGO INRIA
%      load ~/INRIA/MCVEM_publi/Example/logo.dat;
%      ObsPict=flipud(logo);
%      clear logo;
%      load ~/INRIA/MCVEM_publi/Example/logo.true; % logo coded with \{0,1\}
%      TruePict=flipud(logo)+1;
%      clear logo;
%      G=2;
%    %  ObsPict=ObsPict(1:64,1:64);
%    %  TruePict=TruePict(1:64,1:64);
%    NbrNeigh='8-Neigh';
%         %% Deux fichiers d'initialisation :
%         %%  Logos1.init avec moy = [32.3 250] et std=[125 281]
%         %%  LogohistK2.dat avec moy =[57.3 245] et std = [138 301]
      


%     load ~/INRIA/MCVEM_publi/Simulations/Potts2/pottsk2b078001.dat
%     ObsPict=reshape(pottsk2b078001,100,100);
%     clear pottsk2b078001
%     load ~/INRIA/MCVEM_publi/Simulations/Potts2/pottsk2b078001.pf
%     TruePict=reshape(pottsk2b078001,100,100);
%     TruePict=TruePict'+1;
%     clear pottsk2b078001
%     load ~/INRIA/MCVEM_publi/Simulations/Potts2/pottsk2b078001.init
%     Partition=pottsk2b078001;
%     clear pottsk2b078001
%     G=2;
%     NbrNeigh='4-Neigh';
%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
[I J]=size(ObsPict);      % I : Nbr of rows and J : Nbr of columns.
%   Iterations of MCVEM
Stop=0; % initialization of the stopping criterion for MCVEM loops
%NbrIterMax=100; % maximal number of MCVEM iterations
nn=1;   % initialize the number of iterations
Tol=0.0005;   % Tolerance for the stopping criterion


%% Allocate the matrices
Beta=zeros(1,NbrIterMax);
Mu=zeros(G,NbrIterMax);
Sigma2=zeros(G,NbrIterMax);
NbrIterQ=zeros(1,NbrIterMax);
DiagCvgQ=zeros(1,NbrIterMax);
DiagCvgNR=zeros(1,NbrIterMax);
Error=zeros(1,NbrIterMax);

%   Number of simulations
    %NbrSimulMin=500; % Minimal number of simulations at each iteration
    %BurnIn=ceil(0.4*max(NbrSimulMin,ceil(0.5*(1:NbrIterMax).^1.001)));
    %NbrSimul=ceil(max(ceil(((1:NbrIterMax)).^1.001),0.4*NbrSimulMin));% choose the evolution of the number of simulations
MethSimul='PriorGibbs';  % Method for the simulation part : either 'PriorGibbs' or 'ApproxPrior' or 'PriorSW'

% Radius for the local optimization in Beta
Gamma=0.005*ones(1,NbrIterMax);

%   Stabilization procedure (wrt the Beta-component)
NbrProj=0;  %  Initialize the counter of projections.
BetaCenter=1;  %   Center of the compact balls.
Rad=0.8;    %  Radius of the compact C^0, centered at 'BetaCenter'.
DeltaRad=0.1; % Variation of the radius between two consecutive compact balls.




    %%  b/  Initialisation of the parameters
    %%  based on k-means for (Mu,Sigma2)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    disp(sprintf('Initialisation :'));
    %   Run the k-means procedure : initialize "Mean", "Variance" 
    PictCol=reshape(ObsPict,I*J,1);    % matrix IJ x 1
        %Partition=kmeans(PictCol,G);    % matrix IJ x 1
    
    
    for gg=1:G,
        ThetaOld(gg,1)=mean(PictCol(Partition==gg));    % matrix G x 1, for the "Mean"
        ThetaOld(gg,2)=var(PictCol(Partition==gg)); % matrix G x 1, for the "Variance"
    end;
     
    %   Sort the class in the ascending order of the initial "Mean"
    [ThetaOld xx]=sort(ThetaOld);
    PartitionBis=zeros(I*J,1);
    for gg=1:G,
        PartitionBis=gg*(Partition==xx(gg))+PartitionBis;
    end
    Partition=PartitionBis;

    %%  Initialisation of a G-color image,
    %%  based on the partition obtained by k-means
    Zinit=reshape(Partition,I,J);   % matrix I x J, {1, ..., G}-valued.
    FirstZinit=Zinit;
    QdistInit=zeros(I,J,G);
    for gg=1:G,
        QdistInit(:,:,gg)=(Zinit==gg);   % matrix I x J x G, Q distribution at each site (i,j).
    end;
    FirstQdistOld=QdistInit;
    QdistOld=QdistInit;
    
    %   Initialize Beta at random in the first compact ball C^0.
        %BetaOld=BetaCenter+sign(randn(1,1))*rand(1,1)*Rad;
    BetaOld=1;
  
    % Stock the initial values of the parameter
        Mu(:,1)=ThetaOld(:,1);
        Sigma2(:,1)=ThetaOld(:,2);
        Beta(1)=BetaOld;
    
   %   Display the initial values
    disp(sprintf('\t \t Mean %s',Mu(:,1)));
    disp(sprintf('\t \t Std Dev %s',sqrt(Sigma2(:,1))));
    disp(sprintf('\t \t Beta %f',Beta(1)));
    
    
    %% Display the G-valued image,  the observation and the initialization.
     figure(2);
     clf;
     subplot(2,2,1);
     Image(TruePict);
     title('Original image')
     subplot(2,2,2);
     Image(ObsPict);
     title('Observed Image')
     subplot(2,2,3);
     Image(FirstZinit);
     title('Initialization')
     drawnow
     


    while Stop==0,
        %%  c/  Update the parameters
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%  STEP 1 : determine the mean field approximated distribution Q
        Zinit=reshape(Partition,I,J);   % matrix I x J, {1, ..., G}-valued.
        FirstZinit=Zinit;
        QdistInit=zeros(I,J,G);
        for gg=1:G,
            QdistInit(:,:,gg)=(Zinit==gg);   % matrix I x J x G, Q distribution at each site (i,j).
        end;
        FirstQdistOld=QdistInit;
        
        out=RelaxationStep1(ObsPict,FirstQdistOld,ThetaOld,BetaOld,0,NbrNeigh);
        QdistNew=out.M; % matrix I x J x G, distribution at each site (i,j).
        disp(sprintf('Iteration %f',nn));
        disp(sprintf('\t \t Step 1 : Determine the Q-distribution'));
        disp(sprintf('\t \t \t : Nbr of iteration %f',out.NbrIter));
        disp(sprintf('\t \t \t : Diag of cvgce %f',out.DiagCvg));
        NbrIterQ(nn)=out.NbrIter;
        DiagCvgQ(nn)=out.DiagCvg;
        
        
        %%  STEP 2a : Update the mean and variances
        for gg=1:G,
            ThetaNew(gg,1)=sum(sum(ObsPict.*QdistNew(:,:,gg)))/(sum(sum(QdistNew(:,:,gg))));
            DD=(ObsPict-ThetaNew(gg,1)).^2;
            ThetaNew(gg,2)=sum(sum(DD.*QdistNew(:,:,gg)))/(sum(sum(QdistNew(:,:,gg))));
        end;
        disp(sprintf('\t \t Step 2a : Update Mean and Variances'));
        disp(sprintf('\t \t \t : Mean %f',ThetaNew(:,1)));
        disp(sprintf('\t \t \t : Std Dev %f',sqrt(ThetaNew(:,2))));
        
        
        %%  STEP 2b : Update Beta
            if nn==1,
            ZinitChain=ceil(rand(I,J)*G);
        else
             ZinitChain=outSimul.Zfinal;
         end;
        outSimul=SimulVarZ(ZinitChain,BetaOld,G,NbrSimul(nn),BurnIn(nn),NbrNeigh,QdistNew,MethSimul,ObsPict,Mu(:,nn),Sigma2(:,nn));    % structure with two fields
                %ZinitProp=outSimul.Zfinal;  % matrix I x J, {1, ..., G}-valued
                % (modif 2nd August : cf below the reason)
        outNR=Dichotomy(QdistNew,outSimul.Energy,outSimul.Energy2,BetaOld,NbrNeigh,Gamma(nn));
        BetaNew=outNR.Beta;
        disp(sprintf('\t \t Step 2b : Update Beta'));
        disp(sprintf('\t \t \t : Nbr Simul (and burn in : %d) : %d',BurnIn(nn),NbrSimul(nn)));
        disp(sprintf('\t \t \t : Cvgce of Dichotomy (1 if OK) : %d',outNR.DiagCvg));
        disp(sprintf('\t \t \t : MeanQ : %f',outNR.MeanQ));
        disp(sprintf('\t \t \t : GradWinit : %f',outNR.GradWinit));
        disp(sprintf('\t \t \t : GradWend : %f',outNR.GradWend));
        disp(sprintf('\t \t \t : Beta %f',BetaNew));
        DiagCvgNR(nn)=outNR.DiagCvg;
        
         
        %%  d/  Stopping rule
        %%  based on the relative variation
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        Var1=max(max(abs((ThetaNew-ThetaOld)./ThetaOld)));
        Var2=abs((BetaNew-BetaOld)./BetaOld);
        Var=max([Var1 Var2]);
        
        if (nn==NbrIterMax),    %((Var<=Tol) | (nn==NbrIterMax))
                Stop=1;
        else
                nn=nn+1;  
        end;
        
        
        %%  e/ Stabilization procedure on Beta
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if norm(BetaNew-BetaCenter)>=Rad,
            NbrProj=NbrProj+1;
            BetaNew=Beta(1);
            ThetaNew=[Mu(:,1) Sigma2(:,1)];
            QdistOld=FirstQdistOld;
            Rad=Rad+DeltaRad;
        end;
        disp(sprintf('\t \t Step 2c : Stabilization procedure'));
        disp(sprintf('\t \t \t : Nbr Projections : %d',NbrProj));
         
        %% f/  Prepare the next iteration
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        ThetaOld=ThetaNew;
        BetaOld=BetaNew;
        QdistOld=QdistNew;
        
        %% g/  Stock the parameters
        %'%%%%%%%%%%%%%%%%%%%%%%%%%%%
        Mu(:,nn)=ThetaNew(:,1);
        Sigma2(:,nn)=ThetaNew(:,2);
        Beta(nn)=BetaNew;

        figure(4);
        if (nn/20)==round(nn/20),
            plot(Beta(1:nn));
        end;
        drawnow
        
        
        
        %%  h/  Restaure the missing data
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        [x HatZ]=max(QdistNew,[],3);
        Error(nn)=sum(sum(HatZ~=TruePict));
        disp(sprintf('\t \t Step 3 : Nbr errors in segmentation : %d / %d (%2.3f) ',Error(nn),I*J,Error(nn)*100/(I*J)));       
        figure(2); clf;
        subplot(2,2,1);
        ImageCol(TruePict);
        title('True Image');
        subplot(2,2,2);
        ImageCol(ObsPict);
        title('Observation')
        subplot(2,2,3);
        ImageCol(FirstZinit);
        title('Initialization');
        subplot(2,2,4);
        ImageCol(HatZ);
        title('Segmentation');
        drawnow
   
        
        
    end;    % w.r.t. the "while"

    


 