function out=RelaxationStep1(Pict,Minit,Theta,Beta,NbrNeigh,NbrIterMax,TolM);

%%  Compute the best approximate of the a posteriori density,
%%  for the Kullback-Leibler divergence.
%%
%%  INPUT 
%%%%%%%%%
%%  Pict    : matrix I x J 
%%              Observed picture.
%%  Minit : matrix I x J x G
%%              For each site, probability of the G labels
%%  Theta, Beta : matrix G x 2 and real
%%              Current values of the parameters
%%  NbrNeigh   :   Integer
%%              Number of neighoors
%%  NbrIterMax : Maximal number of iterations
%%  TolM : positive real
%%          Tolerance for the stopping criterion
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  OUTPUT
%%%%%%%%%%
%%  Structure that contains the field :
%%
%%  out.M   :   matrix I x J x G
%%          contains for each site (i,j), the probability of the label g for the mean
%%          field approximate distribution.
%%  out.DiagCvg :   {0,1}-valued
%%          '1' if convergence of the iterative part.
%%  out.NbrIter :   integer
%%          Number of iterations
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%          CONVENTION ON the POTTS MODEL 
%%
%%      ln p(z; b) =  H(z;b) - ln W(b)
%%      H(z;b) = 0.5 b \sum_k \sum_{j in N(k)} z_k' z_j
%%             = b \sum_{k ~ j} z_k' z_j
%%             = 0.5 b S(z)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


      

%%  1/ Pre-settings
%%%%%%%%%%%%%%%%%%%%
    [I J G]=size(Minit);
    Mu(1:G)=Theta(:,1);
    Var(1:G)=Theta(:,2);
    
%%  2/ Iterative Part
%%%%%%%%%%%%%%%%%%%%%
Stop=0;
nn=1;

figure(2);
subplot(2,2,1);
Image(Minit(:,:,1));
title(sprintf('Iter 1, class 1'));
drawnow
        

%% FIRST LOOP
    switch NbrNeigh
        case '4-Neigh'
            [A BB]=IterPointFixe4(Beta,reshape(Minit,I,J*G),I,J,G,Pict,Mu,Var);  
            Mnew=reshape(A,I,J,G);
            figure(2);
            subplot(2,2,2);
            Image(Mnew(:,:,1));
            title(sprintf('Iter %d, class 1',nn));
            drawnow
        case '8-Neigh'
            [A BB]=IterPointFixe8(Beta,reshape(Minit,I,J*G),I,J,G,Pict,Mu,Var);  
            Mnew=reshape(A,I,J,G);
            figure(2);
            subplot(2,2,2);
            Image(Mnew(:,:,1));
            title(sprintf('Iter %d, class 1',nn));
            drawnow
    end;
        
    
     
    
nn=nn+1;  

if nn>NbrIterMax,
    Stop=1;
end;

Mold=Mnew;
%%  OTHER LOOPS
while Stop==0
    %   a/  Construction of the new M-matrix
    switch NbrNeigh
        case '4-Neigh'
            [A B]=IterPointFixe4(Beta,reshape(Mold,I,J*G),I,J,G,Pict,Mu,Var);  
            Mnew=reshape(A,I,J,G);
            Mold=reshape(B,I,J,G);
            figure(2);
            subplot(2,2,2);
            Image(Mnew(:,:,1));
            title(sprintf('Iter %d, class 1',nn));
            drawnow
        case '8-Neigh'
            [A B]=IterPointFixe8(Beta,reshape(Mold,I,J*G),I,J,G,Pict,Mu,Var);  
            Mnew=reshape(A,I,J,G);
            Mold=reshape(B,I,J,G);
            figure(2);
            subplot(2,2,2);
            Image(Mnew(:,:,1));
            title(sprintf('Iter %d, class 1',nn));
            drawnow
            
    end;
    %   b/  Stop or not the iterative part
    DeltaM=max(max(max(abs((Mnew-Mold)))));
    Stop=max([(DeltaM<=TolM) (nn==NbrIterMax)]);
    
    %   d/ Prepare the next iteration
    nn=nn+1;
    Mold=Mnew;
end;

    

%%  CONCLUDE
    %   Return the M-matrix
    out.M=Mnew;
    clear Mold Mnew
    %   Diagnostic of convergence
    out.DiagCvg=(nn-1<NbrIterMax);  % '1' if convergence
    %   Number of iterations
    out.NbrIter=nn-1; 