function out=SimulVarZ(Zinit,Beta,G,Nbr,BurnIn,NbrNeigh,Qdist,MethSimul,Y,mu,Sigma2);
%%  INPUT
%%%%%%%%%
%%
%%  Zinit   : matrix I x J, {1, ..., G}-valued
%%          Initial state of the random field
%%  Beta    : integer
%%          Current value of the parameter
%%  G   :   integer
%%          Number of labels
%%  Nbr : integer
%%          Number of samples of the Markov field
%%  NbrNeigh    : integer
%%          Number of neighboors ('4-Neigh' or '8-Neigh')
%%  Qdist    : matrix I x J x G
%%          Q-distribution, approx of the a posteriori distribution.
%%  MethSimul : character string
%%          Method of simulation ('PriorGibbsSyst','PriorSW','PriorGibbsNonSyst','PriorGibbsRandScan').
%%  Y : matrix I x J
%%          Observations
%%  mu  : matrix G x 1
%%          Current value of the mean parameter
%%  Sigma 2 : matrix G x 1
%%          Current value of the variance parameter
%%  OUTPUT
%%%%%%%%%%
%%  Structure with fields 
%%  out.Energy  :   0.5*S(z) for all the simulated pictures z.
%%  out.Zfinal :    last state of the Markov field
%%  out.Energy2 : [ln pi(z;Beta) +Cte] /Beta, in the importance sampling
%%  out.Zinit : initial state of the Markov field
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%          CONVENTION ON the POTTS MODEL 
%%
%%      ln p(z; b) = H(z;b) - ln W(b)
%%      H(z;b) = 0.5 b \sum_k \sum_{j in N(k)} z_k' z_j
%%             = b \sum_{k ~ j} z_k' z_j
%%             = 0.5 b S(z)
%%      b > 0.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    


[I J]=size(Zinit);
Energy=zeros(1,Nbr);
Energy2=zeros(1,Nbr);

figure(2);
subplot(2,2,3);
Image(Zinit);
title(sprintf('Simul 1 (before the burn in)'));
drawnow



%%  1- CASE : '4-Neigh'
%%%%%%%%%%%%%%%%%%%%%%%%
switch NbrNeigh
    case '4-Neigh'
        %% 1-1 : Burn In
        switch MethSimul
            case 'PriorGibbsSyst'
                for nn=1:BurnIn,
                    A=SystGibbs4(Beta,Zinit,G,rand*4294967296);
                    Zinit=A;
                    clear A
                end;
            case 'PriorGibbsNonSyst'
                for nn=1:BurnIn,
                    [A B]=NonSystGibbs4(Beta,Zinit,G,rand*4294967296);
                    Zinit=A;
                    if nn==1,
                        out.Zinit=B;
                    end;
                    clear A B
                end;
            case 'PriorGibbsRandom'
                for nn=1:BurnIn,
                    A=RandomGibbs4(Beta,Zinit,floor(I*J/2),G,rand*4294967296);
                    B=Zinit.*(A==(G+1))+A.*(A<=G);
                    Zinit=B;
                    clear B A
                end;
            case 'PriorGibbsRandScan'
                for nn=1:BurnIn,
                    [A B]=RandScanGibbs4(Beta,Zinit,G,rand*4294967296);
                    Zinit=A;
                    if nn==1,
                        out.Zinit=B;                    end;
                    clear B A
                end;
            case 'PriorSW'
                for nn=1:BurnIn,
                    A=SwendsenWang4(Beta,Zinit,I,J,G,rand*4294967296);
                    Zinit=A;
                    clear A 
                end;
            case 'PosteriorGibbs'
                for nn=1:BurnIn,
                    A=PostGibbs4(Beta,Zinit,G,rand*4294967296,Y,mu,Sigma2);
                    Zinit=A;
                    clear A
                end;
        end;

        %%  1-2 : Initialize the energy
        % Enlarge the initial state of the Markov field
        ZinitEnl=zeros(I+2,J+2);
        ZinitEnl(2:I+1,2:J+1)=Zinit;
        ZinitEnl(1,2:J+1)=Zinit(I,:);
        ZinitEnl(I+2,2:J+1)=Zinit(1,:);
        ZinitEnl(2:I+1,1)=Zinit(:,J);
        ZinitEnl(2:I+1,J+2)=Zinit(:,1);
        ZinitEnl(1,1)=Zinit(I,J);
        ZinitEnl(1,J+2)=Zinit(I,1);
        ZinitEnl(I+2,1)=Zinit(1,J);
        ZinitEnl(I+2,J+2)=Zinit(1,1);
    
        % Count the number of neighboors in the same state
        Neigh=zeros(I,J);
        Neigh=(ZinitEnl(3:I+2,2:J+1)==Zinit);   %   Lower
        Neigh=(ZinitEnl(1:I,2:J+1)==Zinit)+Neigh;   %   Upper
        Neigh=(ZinitEnl(2:I+1,1:J)==Zinit)+Neigh;   %   Left
        Neigh=(ZinitEnl(2:I+1,3:J+2)==Zinit)+Neigh; %   Right

        switch MethSimul
            case 'PosteriorGibbs'
                Energy(1)=0.5*sum(sum(Neigh));  % Corresponds to 0.5 S(z)
                Aux=(Y-mu(Zinit)).^2./Sigma2(Zinit);
                Energy2(1)=Energy(1)-0.5*(sum(sum(Aux))+sum(sum(log(Sigma2(Zinit)))))/Beta;  % Corresponds to S(z)
                clear Aux
            otherwise
                Energy(1)=0.5*sum(sum(Neigh));  % Corresponds to 0.5 S(z)
                Energy2(1)=Energy(1);  % Corresponds to 0.5 S(z)
        end;
        clear Neigh;
        
         
        %% 1-3 :  Run the sampler
        AuxEnl=zeros(I+1,J+1);
        for nn=2:Nbr,
            %  Produce a sample of the random field
            switch MethSimul
                case 'PriorGibbsSyst'
                    A=SystGibbs4(Beta,Zinit,G,rand*4294967296);
                case 'PriorGibbsNonSyst'
                    [A B]=NonSystGibbs4(Beta,Zinit,G,rand*4294967296);
                    if ((BurnIn==0) & (nn==2)),
                        out.Zinit=B;
                    end;
                    clear B
                case 'PriorGibbsRandScan'
                    [A B]=RandScanGibbs4(Beta,Zinit,G,rand*4294967296);
                    if ((BurnIn==0) & (nn==2)),
                        out.Zinit=B;
                    end;
                    clear B 
                case 'PriorGibbsRandom'
                    A=RandomGibbs4(Beta,Zinit,floor(I*J/2),G,rand*4294967296);
                    A=(A==(G+1)).*Zinit+(A<=G).*A;
                case 'PriorSW'
                    A=SwendsenWang4(Beta,Zinit,I,J,G,rand*4294967296);
                case 'PosteriorGibbs'
                    A=PostGibbs4(Beta,Zinit,G,rand*4294967296,Y,mu,Sigma2);
            end;
            outMF.Picture=A;
            figure(2);
            subplot(2,2,4);
            if (round(nn/30)==(nn/30))
                Image(outMF.Picture);
                title(sprintf('Simul %d',nn));
                drawnow
            end;
                
            %   Count the numbers of neighboors in the same state
            AuxEnl(2:I+1,2:J+1)=outMF.Picture;
            AuxEnl(1,2:J+1)=outMF.Picture(I,:);
            AuxEnl(I+2,2:J+1)=outMF.Picture(1,:);
            AuxEnl(2:I+1,1)=outMF.Picture(:,J);
            AuxEnl(2:I+1,J+2)=outMF.Picture(:,1);
        
            Neigh=zeros(I,J);
            Neigh=(AuxEnl(3:I+2,2:J+1)==outMF.Picture); % Lower
            Neigh=(AuxEnl(1:I,2:J+1)==outMF.Picture)+Neigh; % Upper
            Neigh=(AuxEnl(2:I+1,1:J)==outMF.Picture)+Neigh; % Left
            Neigh=(AuxEnl(2:I+1,3:J+2)==outMF.Picture)+Neigh;   % Right
    
            % Compute the energy of the new picture
            switch MethSimul
                case 'PosteriorGibbs'
                    Energy(nn)=0.5*sum(sum(Neigh));
                    Aux=(Y-mu(outMF.Picture)).^2./Sigma2(outMF.Picture);
                    Energy2(nn)=Energy(nn)-0.5*(sum(sum(Aux))+sum(sum(log(Sigma2(outMF.Picture)))))/Beta;  % Corresponds to S(z)
                    clear  A Aux
                otherwise
                    Energy(nn)=0.5*sum(sum(Neigh));
                    Energy2(nn)=Energy(nn);
                    clear A 
                end;
            clear Neigh
            
            % Prepare the next iteration 
            Zinit=outMF.Picture;    % matrix I x J, {1, ..., G}-valued.
         end;   % of the loop "for nn"
         
         
        
        
        
%%  2- CASE : '8-Neigh'
%%%%%%%%%%%%%%%%%%%%%%%%
    case '8-Neigh'
        %% 2-1 : Burn In
        switch MethSimul
            case 'PriorGibbsSyst'
                for nn=1:BurnIn,
                    A=SystGibbs8(Beta,Zinit,G,rand*4294967296);
                    Zinit=A;
                end;
           case 'PriorGibbsNonSyst'
                for nn=1:BurnIn,
                    A=NonSystGibbs8(Beta,Zinit,G,rand*4294967296);
                    Zinit=A;
                end;
            case 'PriorSW'
               for nn=1:BurnIn,
                    A=SwendsenWang8(Beta,Zinit,I,J,G,rand*4294967296);
                    Zinit=A;
                    clear A    
                end;
           case 'PosteriorGibbs'
                for nn=1:BurnIn,
                    A=PostGibbs8(Beta,Zinit,G,rand*4294967296,Y,mu,Sigma2);
                    Zinit=A;
                end;
        end;
        
        %%  2-2 Initialize the energy
        % Enlarge the initial state of the Markov field
        ZinitEnl=zeros(I+2,J+2);
        ZinitEnl(2:I+1,2:J+1)=Zinit;
        ZinitEnl(1,2:J+1)=Zinit(I,:);
        ZinitEnl(I+2,2:J+1)=Zinit(1,:);
        ZinitEnl(2:I+1,1)=Zinit(:,J);
        ZinitEnl(2:I+1,J+2)=Zinit(:,1);
        ZinitEnl(1,1)=Zinit(I,J);
        ZinitEnl(1,J+2)=Zinit(I,1);
        ZinitEnl(I+2,1)=Zinit(1,J);
        ZinitEnl(I+2,J+2)=Zinit(1,1);
    
        % Count the number of neighboors in the same state
        Neigh=zeros(I,J);
        Neigh=(ZinitEnl(3:I+2,2:J+1)==Zinit);   %   Lower
        Neigh=(ZinitEnl(1:I,2:J+1)==Zinit)+Neigh;   %   Upper
        Neigh=(ZinitEnl(2:I+1,1:J)==Zinit)+Neigh;   %   Left
        Neigh=(ZinitEnl(2:I+1,3:J+2)==Zinit)+Neigh; %   Right
        Neigh=Neigh+(ZinitEnl(1:I,1:J)==Zinit); % Upper-Left
        Neigh=Neigh+(ZinitEnl(1:I,3:J+2)==Zinit); % Upper-Right
        Neigh=Neigh+(ZinitEnl(3:I+2,1:J)==Zinit); % Lower-Left
        Neigh=Neigh+(ZinitEnl(3:I+2,3:J+2)==Zinit); % Lower-Right
     
        switch MethSimul
            case 'PosteriorGibbs'
                Energy(1)=0.5*sum(sum(Neigh));  % Corresponds to 0.5 S(z)
                Aux=(Y-mu(Zinit))^2./Sigma2(Zinit);
                Energy2(1)=Energy(1)-0.5*(sum(sum(Aux))+sum(sum(log(Sigma2(Zinit)))))/Beta;  % Corresponds to S(z)
                clear Aux
            otherwise
                Energy(1)=0.5*sum(sum(Neigh));  % Corresponds to 0.5 S(z)
                Energy2(1)=Energy(1);  % Corresponds to 0.5 S(z)
            end;
        clear Neigh;
   
        %%  2-3 : Run the sampler
        AuxEnl=zeros(I+1,J+1);
          for nn=2:Nbr,
            %  Produce a sample of the random field
            switch MethSimul
                case 'PriorGibbsSyst'
                    A=SystGibbs8(Beta,Zinit,G,rand*4294967296);
                case 'PriorGibbsNonSyst'
                    A=NonSystGibbs8(Beta,Zinit,G,rand*4294967296);
                case 'PriorSW'
                    A=SwendsenWang8(Beta,Zinit,I,J,G,rand*4294967296);
                case 'PosteriorGibbs'
                    A=PostGibbs8(Beta,Zinit,G,rand*4294967296,Y,mu,Sigma2);
            end;
            outMF.Picture=A;
            
            figure(2);
            subplot(2,2,4);
            if (round(nn/30)==(nn/30))
                Image(outMF.Picture);
                title(sprintf('Simul %d',nn));
                drawnow
            end;
   
            %   Count the numbers of neighboors in the same state
            AuxEnl(2:I+1,2:J+1)=outMF.Picture;
            AuxEnl(1,2:J+1)=outMF.Picture(I,:);
            AuxEnl(I+2,2:J+1)=outMF.Picture(1,:);
            AuxEnl(2:I+1,1)=outMF.Picture(:,J);
            AuxEnl(2:I+1,J+2)=outMF.Picture(:,1);
    
            AuxEnl(1,1)=outMF.Picture(I,J);
            AuxEnl(I+2,J+2)=outMF.Picture(1,1);
            AuxEnl(1,J+2)=outMF.Picture(I,1);
            AuxEnl(I+2,1)=outMF.Picture(1,J);
    
            Neigh=zeros(I,J);
            Neigh=(AuxEnl(3:I+2,2:J+1)==outMF.Picture); % Lower
            Neigh=(AuxEnl(1:I,2:J+1)==outMF.Picture)+Neigh; % Upper
            Neigh=(AuxEnl(2:I+1,1:J)==outMF.Picture)+Neigh; % Left
            Neigh=(AuxEnl(2:I+1,3:J+2)==outMF.Picture)+Neigh; % Right
            Neigh=(AuxEnl(1:I,1:J)==outMF.Picture)+Neigh; % Upper-Left
            Neigh=(AuxEnl(1:I,3:J+2)==outMF.Picture)+Neigh; % Upper-Right
            Neigh=(AuxEnl(3:I+2,1:J)==outMF.Picture)+Neigh; % Lower-Left
            Neigh=(AuxEnl(3:I+2,3:J+2)==outMF.Picture)+Neigh; % Lower-Right
    
            % Stock the  energy of the new picture
            switch MethSimul
                case 'PosteriorGibbs'
                    Energy(nn)=0.5*sum(sum(Neigh));
                    Aux=(Y-mu(outMF.Picture)).^2./Sigma2(outMF.Picture);
                    Energy2(nn)=Energy(nn)-0.5*(sum(sum(Aux))+sum(sum(log(Sigma2(outMF.Picture)))))/Beta;
                    clear A Aux
                otherwise
                    Energy(nn)=0.5*sum(sum(Neigh));
                    Energy2(nn)=Energy(nn);
                    clear A 
            end;
            clear Neigh
            
            % Prepare the next iteration
            Zinit=outMF.Picture;    % matrix I x J, {1, ..., G}-valued.
        end; % of the loop "for nn"
       
        
end;    % of the switch on the nbr of Neighbors



%%  3- DISPLAY the energies
%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure(3);
clf;
subplot(3,1,1);
plot(1:Nbr,Energy(1,:),'r');
hold on;
plot(1:Nbr,Energy2(1,:),'g:');
title('Energy');
drawnow


%%  4- OUTPUT
%%%%%%%%%%%%%%
out.Energy=Energy;  % matrix 1 x Nbr
out.Energy2=Energy2;    % matrix 1 x Nbr
out.Zfinal=outMF.Picture;   % matrix I x J, {1, ..., G}-valued.
out.Zinit=Zinit;

clear Energy Energy2 outMF Neigh AuxEnl ZinitEnl Zinit





