/* This is a mex-file for summing the neighbors of each pixel, in each state.
   If each neighbor is {1, ..., G}-valued, this sum is n1 + n2 5 + ...+ nG 5^{G-1}
   where nk is the number of neighbors in state k.
   The distribution of this sum over the IxJ pixels of the picture is returned.

   $Id: CountNeigh4.c, v.1 12/11/2005$     */

/*
mex -v -lgsl -lgslcblas CountNeigh4.c   */

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include "mex.h"
#include <math.h>
#include <stdio.h>
#define square(x) ((x) * (x))

/***************************************/
/* LIST of the (auxiliary) FUNCTIONS   */
/***************************************/

/* Return the neighbors of the pixel (ii,jj) in a 4-connected topology */
void  NeighIndex4(int ii, int jj, int I, int J, int *x); 



/******************/
/*  MAIN FUNCTION */
/******************/
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
  double *S, *Pict,*Gaux,*x,Sum;
  int I,J,G,M,ii,jj,gg,NeighIndex[4],IntSum;


 /*  Check for proper number of input arguments */
  if (nrhs != 3)
    mexErrMsgTxt("Three inputs required.");

 /* Get input arguments (not checking size on b...) */
  S = mxGetPr(prhs[0]);
  M = mxGetM(prhs[0]);
  Pict = mxGetPr(prhs[1]);
  I = mxGetM(prhs[1]);
  J = mxGetN(prhs[1]);
  Gaux = mxGetPr(prhs[2]);

  G=(int) (*Gaux);

  /* Set the output pointer to the output matrix */
  plhs[0] = mxCreateDoubleMatrix(M,1,mxREAL);
  x=mxGetPr(plhs[0]);

  for (ii=0; ii<M; ii++)
    {
      *(x+ii)=*(S+ii);
    }
	 
 for (ii=1; ii<=I; ii++)
    {
      for (jj=1; jj<=J; jj++)
	{
	  /* find its 4 neighbors */
	  NeighIndex4(ii,jj,I,J,NeighIndex);
	  Sum=0;
	  for (gg=0;gg<4;gg++)
	    {
	      /* Consider in turn each neighbor */
	      Sum=Sum+pow(5,*(Pict+NeighIndex[gg])-1);
	    }
	   IntSum=(int) Sum;
	   *(x+IntSum)=*(x+IntSum)+1;
	}
    }
}

/****************************************/
/* CODE of the  (auxiliary) FUNCTIONS   */
/****************************************/

void NeighIndex4(int ii, int jj,int I, int J, int *x)
{
  /* It is important to sort the neighbors with the convention : west, north, east, south */
  
  if(jj==1)
    { 
      if (ii==1)
	{ 
	  *x=(J-1)*I+1-1; 
	  *(x+1)=(1-1)*I+I-1;
	  *(x+2)=(2-1)*I+1-1;
	  *(x+3)=(1-1)*I+2-1;
	}
      else if (ii==I)
	{
	  *x=(J-1)*I+I-1;
	  *(x+1)=(1-1)*I+I-1-1;
	  *(x+2)=(2-1)*I+I-1;
	  *(x+3)=(1-1)*I+1-1;
	}
      else
	{
	  *x=(J-1)*I+ii-1;
	  *(x+1)=(1-1)*I+ii-1-1;
	  *(x+2)=(2-1)*I+ii-1;
	  *(x+3)=(1-1)*I+ii+1-1; 
	}
    }
  else if (jj==J)
    {
      if (ii==1)
	{ 
	 *x=(J-1-1)*I+1-1;
	 *(x+1)=(J-1)*I+I-1;
	 *(x+2)=(1-1)*I+1-1;
	 *(x+3)=(J-1)*I+2-1;
	}
      else if (ii==I)
	{
	  *x=(J-1-1)*I+I-1;
	  *(x+1)=(J-1)*I+I-1-1;
	  *(x+2)=(1-1)*I+I-1;
	  *(x+3)=(J-1)*I+1-1;
	}
      else
	{
	 *x=(J-1-1)*I+ii-1;
	 *(x+1)=(J-1)*I+ii-1-1;
         *(x+2)=(1-1)*I+ii-1;
         *(x+3)=(J-1)*I+ii+1-1;
	}
    }
  else
    { if (ii==1)
      {
	*x=(jj-1-1)*I+1-1;
	*(x+1)=(jj-1)*I+I-1;
	*(x+2)=(jj+1-1)*I+1-1;
	*(x+3)=(jj-1)*I+2-1;
      }
    else if (ii==I)
      {
	*x=(jj-1-1)*I+I-1;
	*(x+1)=(jj-1)*I+I-1-1;
        *(x+2)=(jj+1-1)*I+I-1;
        *(x+3)=(jj-1)*I+1-1;
      }
    else
      { 
	*x=(jj-1-1)*I+ii-1;
	*(x+1)=(jj-1)*I+ii-1-1;
	*(x+2)=(jj+1-1)*I+ii-1;
	*(x+3)=(jj-1)*I+ii+1-1;
      }
    }
}




