/* This is a mex-file for solving the mean field fixed point equation
   $Id: IterPointFixe4.c, v.1 05/10/2005$     */




/* Called with the arguments
   Beta : positive real 
   M : matrix I x (JG) (G blocks of size (IxJ))
   I,J,G : positive integers
   Pict : matrix I x J
   Mu : matrix G x 1
   Sigma2 : matrix G x 1
  
*/

/*
mex -v -lgsl -lgslcblas IterPointFixe4.c
 */


/* When using generators other than uniform be sure to include the TWO gsl
   files below!!! */
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include "mex.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>


#define square(x) ((x) * (x))

/***************************************/
/* LIST of the (auxiliary) FUNCTIONS   */
/***************************************/

/* Return the neighbors of the pixel (ii,jj) in a 4-connected topology */
void  NeighIndex4(int ii, int jj, int I, int J, int *x); 



/******************/
/*  MAIN FUNCTION */
/******************/


/* Everything is done in the gateway routine */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
  double *Beta , *M, *Pict, *Mu, *Sigma2, bb, *Gaux, *Iaux, *Jaux, *Arg, *cumul, *Sum,  *x, *y;
  int  I,J,G,ii,jj,gg,n,NeighIndex[4];
  
  /*  Check for proper number of input arguments */
  if (nrhs != 8)
    mexErrMsgTxt("Eight inputs required.");



  /* Get input arguments (not checking size on b...) */
  Beta = mxGetPr(prhs[0]);
  M = mxGetPr(prhs[1]);
  Iaux = mxGetPr(prhs[2]);
  Jaux= mxGetPr(prhs[3]);
  Gaux = mxGetPr(prhs[4]);
  Pict=mxGetPr(prhs[5]);
  Mu=mxGetPr(prhs[6]);
  Sigma2=mxGetPr(prhs[7]);

  bb=*Beta;
  G=(int) (*Gaux);
  I=(int) (*Iaux);
  J=(int) (*Jaux);

 
  
  
  Arg=malloc(G*sizeof(double));
  cumul=malloc((G+1)*sizeof(double)); 
  Sum=malloc(G*sizeof(double)); 


  /* Set the output pointer to the output matrix */
  plhs[0] = mxCreateDoubleMatrix(I,J*G,mxREAL);
  plhs[1] = mxCreateDoubleMatrix(I,J*G,mxREAL);
  /* Create a C pointer to a copy of the output matrix */
  x = mxGetPr(plhs[0]);
  y = mxGetPr(plhs[1]);
 


  /* Loop on all the columns,
     and 
     on all the rows
  */

  for (jj=J; jj>=1; jj--)
    {
      for (ii=1; ii<=I; ii++)
	{
	  /* find its 4 neighbors */
	  NeighIndex4(ii,jj,I,J,NeighIndex);
	  /* compute the function to iterate */
	  cumul[0]=0;
	  for (gg=1; gg<=G; gg++)
	    {
	      Sum[gg-1]=*(M+NeighIndex[0]+(I*J)*(gg-1))+*(M+NeighIndex[1]+(I*J)*(gg-1))+*(M+NeighIndex[2]+(I*J)*(gg-1))+*(M+NeighIndex[3]+(I*J)*(gg-1));
	      Arg[gg-1]=square(*(Pict+(jj-1)*I+ii-1)-*(Mu+gg-1))/(*(Sigma2+gg-1))-2*bb*Sum[gg-1];
	      *(y+(jj-1)*I+ii-1+(I*J)*(gg-1))=*(M+(jj-1)*I+ii-1+(I*J)*(gg-1));
	      *(M+(jj-1)*I+ii-1+(I*J)*(gg-1))=exp(-0.5*Arg[gg-1])/sqrt(*(Sigma2+gg-1));
	      cumul[gg]=cumul[gg-1]+*(M+(jj-1)*I+ii-1+(I*J)*(gg-1));
	    }
	  /* Normalize */
	  for (gg=1; gg<=G; gg++)
	    {
	      *(M+(jj-1)*I+ii-1+(I*J)*(gg-1))=*(M+(jj-1)*I+ii-1+(I*J)*(gg-1))/cumul[G];
	      *(x+(jj-1)*I+ii-1+(I*J)*(gg-1))=*(M+(jj-1)*I+ii-1+(I*J)*(gg-1));
	    }
	}
    }

  
  free(Arg);
  free(cumul);
  free(Sum);
 
}



/****************************************/
/* CODE of the  (auxiliary) FUNCTIONS   */
/****************************************/

void NeighIndex4(int ii, int jj,int I, int J, int *x)
{
  /* It is important to sort the neighbors with the convention : west, north, east, south */
  
  if(jj==1)
    { 
      if (ii==1)
	{ 
	  *x=(J-1)*I+1-1; 
	  *(x+1)=(1-1)*I+I-1;
	  *(x+2)=(2-1)*I+1-1;
	  *(x+3)=(1-1)*I+2-1;
	}
      else if (ii==I)
	{
	  *x=(J-1)*I+I-1;
	  *(x+1)=(1-1)*I+I-1-1;
	  *(x+2)=(2-1)*I+I-1;
	  *(x+3)=(1-1)*I+1-1;
	}
      else
	{
	  *x=(J-1)*I+ii-1;
	  *(x+1)=(1-1)*I+ii-1-1;
	  *(x+2)=(2-1)*I+ii-1;
	  *(x+3)=(1-1)*I+ii+1-1; 
	}
    }
  else if (jj==J)
    {
      if (ii==1)
	{ 
	 *x=(J-1-1)*I+1-1;
	 *(x+1)=(J-1)*I+I-1;
	 *(x+2)=(1-1)*I+1-1;
	 *(x+3)=(J-1)*I+2-1;
	}
      else if (ii==I)
	{
	  *x=(J-1-1)*I+I-1;
	  *(x+1)=(J-1)*I+I-1-1;
	  *(x+2)=(1-1)*I+I-1;
	  *(x+3)=(J-1)*I+1-1;
	}
      else
	{
	 *x=(J-1-1)*I+ii-1;
	 *(x+1)=(J-1)*I+ii-1-1;
         *(x+2)=(1-1)*I+ii-1;
         *(x+3)=(J-1)*I+ii+1-1;
	}
    }
  else
    { if (ii==1)
      {
	*x=(jj-1-1)*I+1-1;
	*(x+1)=(jj-1)*I+I-1;
	*(x+2)=(jj+1-1)*I+1-1;
	*(x+3)=(jj-1)*I+2-1;
      }
    else if (ii==I)
      {
	*x=(jj-1-1)*I+I-1;
	*(x+1)=(jj-1)*I+I-1-1;
        *(x+2)=(jj+1-1)*I+I-1;
        *(x+3)=(jj-1)*I+1-1;
      }
    else
      { 
	*x=(jj-1-1)*I+ii-1;
	*(x+1)=(jj-1)*I+ii-1-1;
	*(x+2)=(jj+1-1)*I+ii-1;
	*(x+3)=(jj-1)*I+ii+1-1;
      }
    }
}

