/* This is a mex-file for generating a Markov chain by the Gibbs sampler (deterministic scan, in staggered rows)
   with invariant distribution a Potts distribution.
   The random seed is passed as an additionnal argument.

   $Id: NonSystGibbs4.c, v.1 30/09/2005$     */

/*
mex -v -lgsl -lgslcblas NonSystGibbs4.c   */


/* INPUT VARIABLES : 
   -----------------
   Beta : positive integer
   Pict : matrix I x J
   G : positive integer >=2
   A seed for the random generator.
*/



#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include "mex.h"
#include <math.h>
#include <stdio.h>
/***************************************/
/* LIST of the (auxiliary) FUNCTIONS   */
/***************************************/

/* Return the neighbors of the pixel (ii,jj) in a 4-connected topology */
void  NeighIndex4(int ii, int jj, int I, int J, int *x); 



/******************/
/*  MAIN FUNCTION */
/******************/

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
  double *Beta, *Pict, *x, *y,  bb, *Proba,  normalisation, alea, *cumul, *Gaux, ggg;
  int ii, iii,jj, gg, I, J,G, n, *Neigh , NeighIndex[4],Imax1, Imax2;
  extern unsigned long int gsl_rng_default_seed;
  extern const gsl_rng_type *gsl_rng_default;
  gsl_rng *r;


  /*  Check for proper number of input arguments */
  if (nrhs != 4)
    mexErrMsgTxt("Four inputs required.");


  /* Get input arguments (not checking size on b...) */
  Beta = mxGetPr(prhs[0]);
  Pict = mxGetPr(prhs[1]);
  I = mxGetM(prhs[1]);
  J = mxGetN(prhs[1]);
  Gaux = mxGetPr(prhs[2]);
  gsl_rng_default_seed = (unsigned long int) mxGetScalar(prhs[3]);

  bb=*Beta;
  G=(int) (*Gaux);
  
  Neigh=malloc(G*sizeof(int));
  Proba=malloc(G*sizeof(double));
  cumul=malloc(G*sizeof(double));

  /* Set the output pointer to the output matrix */
  plhs[0] = mxCreateDoubleMatrix(I,J,mxREAL);
  plhs[1] = mxCreateDoubleMatrix(I,J,mxREAL);
/* Create a C pointer to a copy of the output matrix */
  x = mxGetPr(plhs[0]);
  y = mxGetPr(plhs[1]);

  r=gsl_rng_alloc(gsl_rng_default);



  if (I==2*floor(I/2))
    {
      Imax1=0.5*I-1;
      Imax2=0.5*I;
    }
  if (I==2*floor(I/2)+1)
    {
      Imax1=0.5*(I-1);
      Imax2=Imax1;
    }


/* Loop on all the columns,
     and 
   on all the rows
*/
 
   for (jj=1; jj<=J; jj++)
     { 
      if (jj==2*floor(jj/2))   /* if jj is even */
	{ for (iii=0; iii<=Imax1; iii++) /* change the odd rows */
	    { ii=1+2*iii;
	      /* find its 4 neighbors */
	      NeighIndex4(ii,jj,I,J,NeighIndex);
	      /* compute the conditional probability */
	      for (gg=1; gg<=G; gg++)
		{ 
		  ggg=(double) gg;
		  Neigh[gg-1]=(*(Pict+NeighIndex[0])==ggg)+(*(Pict+NeighIndex[1])==ggg)+(*(Pict+NeighIndex[2])==ggg)+(*(Pict+NeighIndex[3])==ggg);
		  Proba[gg-1]=exp(bb*Neigh[gg-1]);
		} /* end of loop "for gg" */
	      /* Normalise the conditional probability */
	      cumul[0] = Proba[0];
	      for ( n=1; n<G; n++ )
		{
		  cumul[n] = cumul[n-1] + Proba[n];
		}
	      normalisation = cumul[G-1];
	      /* Sample a uniform random variable */
	      alea = gsl_rng_uniform(r)*normalisation;
	      /* Choose the new color of the piwel (ii,jj) */
	      n=0;
	      while ((alea > cumul[n]) & (n<=(G-1)))
		{
		  n++;
		}
	      *(y+(jj-1)*I+ii-1)=*(Pict+(jj-1)*I+ii-1);
	      /* Update the state of the Markov chain */
	      *(Pict+(jj-1)*I+ii-1)=(double)(n+1);
	      /* Stock the output picture */
	      *(x+(jj-1)*I+ii-1)=(double)(n+1);
	    } /* end of loop "for iii" */
	} /* end of test "if jj is even" */
      
      
      if (jj==2*floor(jj/2)+1)   /* if jj is odd */
	{
	  for (iii=1; iii<=Imax2; iii++) /* change the even rows */
	    {
	      ii=2*iii;
	      /* find its 4 neighbors */
	      NeighIndex4(ii,jj,I,J,NeighIndex);
	      /* compute the conditional probability */
	      for (gg=1; gg<=G; gg++)
		{ 
		  ggg=(double) gg;
		  Neigh[gg-1]=(*(Pict+NeighIndex[0])==ggg)+(*(Pict+NeighIndex[1])==ggg)+(*(Pict+NeighIndex[2])==ggg)+(*(Pict+NeighIndex[3])==ggg);
		  Proba[gg-1]=exp(bb*Neigh[gg-1]);
		} /* end of loop "for gg" */
	      /* Normalise the conditional probability */
	      cumul[0] = Proba[0];
	      for ( n=1; n<G; n++ )
		{
		  cumul[n] = cumul[n-1] + Proba[n];
		}
	      normalisation = cumul[G-1];
	      /* Sample a uniform random variable */
	      alea = gsl_rng_uniform(r)*normalisation;
	      /* Choose the new color of the piwel (ii,jj) */
	      n=0;
	      while ((alea > cumul[n]) & (n<=(G-1)))
		{
		  n++;
		}
	      *(y+(jj-1)*I+ii-1)=*(Pict+(jj-1)*I+ii-1);
	      /* Update the state of the Markov chain */
	      *(Pict+(jj-1)*I+ii-1)=(double)(n+1);
	      /* Stock the output picture */
	      *(x+(jj-1)*I+ii-1)=(double)(n+1);
	    } /* end of loop "for iii" */
	} /* end of test "if jj is odd" */
    } /* end of loop "for jj" */




  for (jj=1; jj<=J; jj++)
     { 
      if (jj==2*floor(jj/2)+1)   /* if jj is odd */
	{ for (iii=0; iii<=Imax1; iii++) /* change the odd rows */
	    { ii=1+2*iii;
	      /* find its 4 neighbors */
	      NeighIndex4(ii,jj,I,J,NeighIndex);
	      /* compute the conditional probability */
	      for (gg=1; gg<=G; gg++)
		{ 
		  ggg=(double) gg;
		  Neigh[gg-1]=(*(Pict+NeighIndex[0])==ggg)+(*(Pict+NeighIndex[1])==ggg)+(*(Pict+NeighIndex[2])==ggg)+(*(Pict+NeighIndex[3])==ggg);
		  Proba[gg-1]=exp(bb*Neigh[gg-1]);
		} /* end of loop "for gg" */
	      /* Normalise the conditional probability */
	      cumul[0] = Proba[0];
	      for ( n=1; n<G; n++ )
		{
		  cumul[n] = cumul[n-1] + Proba[n];
		}
	      normalisation = cumul[G-1];
	      /* Sample a uniform random variable */
	      alea = gsl_rng_uniform(r)*normalisation;
	      /* Choose the new color of the piwel (ii,jj) */
	      n=0;
	      while ((alea > cumul[n]) & (n<=(G-1)))
		{
		  n++;
		}
	      *(y+(jj-1)*I+ii-1)=*(Pict+(jj-1)*I+ii-1);
	      /* Update the state of the Markov chain */
	      *(Pict+(jj-1)*I+ii-1)=(double)(n+1);
	      /* Stock the output picture */
	      *(x+(jj-1)*I+ii-1)=(double)(n+1);
	    } /* end of loop "for iii" */
	} /* end of test "if jj is odd" */
      
      
      if (jj==2*floor(jj/2))   /* if jj is even */
	{
	  for (iii=1; iii<=Imax2; iii++) /* change the even rows */
	    {
	      ii=2*iii;
	      /* find its 4 neighbors */
	      NeighIndex4(ii,jj,I,J,NeighIndex);
	      /* compute the conditional probability */
	      for (gg=1; gg<=G; gg++)
		{ 
		  ggg=(double) gg;
		  Neigh[gg-1]=(*(Pict+NeighIndex[0])==ggg)+(*(Pict+NeighIndex[1])==ggg)+(*(Pict+NeighIndex[2])==ggg)+(*(Pict+NeighIndex[3])==ggg);
		  Proba[gg-1]=exp(bb*Neigh[gg-1]);
		} /* end of loop "for gg" */
	      /* Normalise the conditional probability */
	      cumul[0] = Proba[0];
	      for ( n=1; n<G; n++ )
		{
		  cumul[n] = cumul[n-1] + Proba[n];
		}
	      normalisation = cumul[G-1];
	      /* Sample a uniform random variable */
	      alea = gsl_rng_uniform(r)*normalisation;
	      /* Choose the new color of the piwel (ii,jj) */
	      n=0;
	      while ((alea > cumul[n]) & (n<=(G-1)))
		{
		  n++;
		}
	      *(y+(jj-1)*I+ii-1)=*(Pict+(jj-1)*I+ii-1);
	      /* Update the state of the Markov chain */
	      *(Pict+(jj-1)*I+ii-1)=(double)(n+1);
	      /* Stock the output picture */
	      *(x+(jj-1)*I+ii-1)=(double)(n+1);
	    } /* end of loop "for iii" */
	} /* end of test "if jj is even" */
    } /* end of loop "for jj" */








      free(Neigh);
      free(Proba);
      free(cumul);
       /* Free the memory allocated for the GSL generator */
      gsl_rng_free(r);
}







/****************************************/
/* CODE of the  (auxiliary) FUNCTIONS   */
/****************************************/

void NeighIndex4(int ii, int jj,int I, int J, int *x)
{
  /* It is important to sort the neighbors with the convention : west, north, east, south */
  
  if(jj==1)
    { 
      if (ii==1)
	{ 
	  *x=(J-1)*I+1-1; 
	  *(x+1)=(1-1)*I+I-1;
	  *(x+2)=(2-1)*I+1-1;
	  *(x+3)=(1-1)*I+2-1;
	}
      else if (ii==I)
	{
	  *x=(J-1)*I+I-1;
	  *(x+1)=(1-1)*I+I-1-1;
	  *(x+2)=(2-1)*I+I-1;
	  *(x+3)=(1-1)*I+1-1;
	}
      else
	{
	  *x=(J-1)*I+ii-1;
	  *(x+1)=(1-1)*I+ii-1-1;
	  *(x+2)=(2-1)*I+ii-1;
	  *(x+3)=(1-1)*I+ii+1-1; 
	}
    }
  else if (jj==J)
    {
      if (ii==1)
	{ 
	 *x=(J-1-1)*I+1-1;
	 *(x+1)=(J-1)*I+I-1;
	 *(x+2)=(1-1)*I+1-1;
	 *(x+3)=(J-1)*I+2-1;
	}
      else if (ii==I)
	{
	  *x=(J-1-1)*I+I-1;
	  *(x+1)=(J-1)*I+I-1-1;
	  *(x+2)=(1-1)*I+I-1;
	  *(x+3)=(J-1)*I+1-1;
	}
      else
	{
	 *x=(J-1-1)*I+ii-1;
	 *(x+1)=(J-1)*I+ii-1-1;
         *(x+2)=(1-1)*I+ii-1;
         *(x+3)=(J-1)*I+ii+1-1;
	}
    }
  else
    { if (ii==1)
      {
	*x=(jj-1-1)*I+1-1;
	*(x+1)=(jj-1)*I+I-1;
	*(x+2)=(jj+1-1)*I+1-1;
	*(x+3)=(jj-1)*I+2-1;
      }
    else if (ii==I)
      {
	*x=(jj-1-1)*I+I-1;
	*(x+1)=(jj-1)*I+I-1-1;
        *(x+2)=(jj+1-1)*I+I-1;
        *(x+3)=(jj-1)*I+1-1;
      }
    else
      { 
	*x=(jj-1-1)*I+ii-1;
	*(x+1)=(jj-1)*I+ii-1-1;
	*(x+2)=(jj+1-1)*I+ii-1;
	*(x+3)=(jj-1)*I+ii+1-1;
      }
    }
}


